//***************************************************************************
// Group Linslot / Linux - Slotrace Manager
// File setup.hpp
// Date 05.12.06 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

#ifndef _SETUP_H_
#define _SETUP_H_

//***************************************************************************
// Includes
//***************************************************************************

#include <QDialog>
#include <QSettings>

#include <ui_setup.h>

//***************************************************************************
// Class LinslotWindow
//***************************************************************************

class SetupDialog : public QDialog, public Ui::SetupDialog
{
      Q_OBJECT
      
   public:
      
      // object
      
      SetupDialog(QString configPath);
      ~SetupDialog();

      // gettings

      int getRoundCount()           { return roundCount; }
      int getRoundLimit()           { return roundLimit; }
      double getSlotLength()        { return slotLength; }
      const char* getUsbDevice()    { return usbDevice; }
      int getMaxTimeRoundRace()     { return maxTimeRoundRace; }
      int getMaxTimeTraining()      { return maxTimeTraining; }
      int getSpeedFactor()          { return speedFactor; }
      const char* getCourseName()   { return courseName; }
      const char* getDatabaseName() { return databaseName; }

      QSettings* getSettings()      { return settings; }
      int getAbortAtJumpTheGun()    { return abortAtJumpTheGun; }
      int getPenaltyAtJumpTheGun()  { return penaltyAtJumpTheGun; }

   protected:
      
      QSettings* settings;
      int roundCount;              // round limit for races
      int roundLimit;              // round limit for training
      double slotLength;
      int maxTimeRoundRace;
      int maxTimeTraining;
      int speedFactor;
      char courseName[100];
      char usbDevice[100];
      char databaseName[100];
      int abortAtJumpTheGun;
      int penaltyAtJumpTheGun;

   private slots:

      void on_tabWidget_currentChanged(int tab);
      void on_pushButtonAddDriver_clicked();
      void on_pushButtonDelDriver_clicked();
      void on_spinBoxRoundCount_valueChanged(int value);
      void on_spinBoxTrainingRoundLimit_valueChanged(int value);
      void on_spinBoxJumpTheGunPenaltyTime_valueChanged(int value);
      void on_doubleSpinBoxSlotLength_valueChanged(double value);
      void on_comboBoxDevice_currentIndexChanged(const QString text);
      void on_comboBoxDevice_editTextChanged(const QString text);
      void on_lineEditCourse_editingFinished();
      void on_lineEditDbName_editingFinished();
      void on_timeEditRoundRace_editingFinished();
      void on_timeEditTraining_editingFinished();
      void on_radioButtonFaktor1_toggled(bool checked);
      void on_radioButtonFaktor32_toggled(bool checked);
      void on_radioButtonFaktor24_toggled(bool checked);
      void on_radioButtonJumpTheGunAbort_toggled(bool checked);
      void on_radioButtonJumpTheGunPenalty_toggled(bool checked);

};

//***************************************************************************
#endif // _SETUP_H_
